@@ -709,7 +709,10 @@ class CommonIdentityTests(object):
         resps = [{'json': disc}, {'status_code': 500}]
 
         # We should only try to fetch the unversioned non-project_id url once
-        self.requests_mock.get(self.TEST_VOLUME.unversioned.public, resps)
+        # Because the catalog has the versioned endpoint but we constructed
+        # an unversioned endpoint, the url needs to have a trailing /
+        self.requests_mock.get(
+            self.TEST_VOLUME.unversioned.public + '/', resps)
 
         # Fetch v2.0 first - since that doesn't match endpoint optimization,
         # it should fetch the unversioned endpoint
@@ -854,6 +857,32 @@ class CommonIdentityTests(object):
         self.assertEqual(a_token, b_token)
         self.assertAccessInfoEqual(a.auth_ref, b.auth_ref)
 
+    def test_pathless_url(self):
+        disc = fixture.DiscoveryList(v2=False, v3=False)
+        url = 'http://path.less.url:1234'
+        disc.add_microversion(href=url, id='v2.1')
+
+        self.stub_url('GET', base_url=url, status_code=200, json=disc)
+
+        token = fixture.V2Token()
+        service = token.add_service('network')
+        service.add_endpoint(public=url, admin=url, internal=url)
+
+        self.stub_url('POST', ['tokens'], base_url=url, json=token)
+
+        v2_auth = identity.V2Password(url, username='u', password='p')
+
+        sess = session.Session(auth=v2_auth)
+
+        data = sess.get_endpoint_data(service_type='network')
+
+        # Discovery ran and returned the URL
+        self.assertEqual(url, data.url)
+
+        # Run with a project_id to ensure that path is covered
+        self.assertEqual(
+            3, len(list(data._get_discovery_url_choices(project_id='42'))))
+
 
 class V3(CommonIdentityTests, utils.TestCase):
 
